
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : david.scm
;; DESCRIPTION : tools by David but which Joris does not like (yet)
;; COPYRIGHT   : (C) 2002  Joris van der Hoeven, David Allouche
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define (flatten tree)
  "Flatten a tree-like structure made of lists to a simple list."
  (define (flatten* tree bt res)
    (if (null? tree)
	(if (null? bt)
	    res
	    (flatten* (car bt) (cdr bt) res))
	(if (pair? (car tree))
	    (flatten* (car tree) (cons (cdr tree) bt) res)
	    (flatten* (cdr tree) bt (cons (car tree) res)))))
  (reverse (flatten* tree () ())))

(define (list->alist dscs lst)
  "Parses the list lst into an association list according to dscs.

dscs is a list of <descriptor> with the following syntax:
<descriptor> := (<key> <pred?> [<lam>])
<key> is an arbitrary object (probably a symbol, to use assq).
<pred?> and <lam> are one-argument functions.

The first <descriptor> whose <pred?> matches an item of lst is used to
produce an association whose car is <key> and whose cdr is
(<lam> lst). The <lam> element defaults to identity.

If an item of lst is faulty and matches no <pred?>, the result is
(list #t idx) where idx is the index of the faulty item. Otherwise,
the conversion succeds and the result is (list #f alist)."

  (define (iterate-dscs dscs x)
    (if (null? dscs)
	#f
	(let ((tag (caar dscs))
	      (pred? (cadar dscs))
	      (func (if (null? (cddar dscs))
			(lambda (x) x)
			(caddar dscs))))
	  (if (pred? x)
	      (cons tag (func x))
	      (iterate-dscs (cdr dscs) x)))))
  (define (iterate-lst dscs lst res i)
    (if (null? lst)
	(list #f res)
	(let ((res2 (iterate-dscs dscs (car lst))))
	  (if (not res2)
	      (list #t i)
	      (iterate-lst dscs (cdr lst) (cons res2 res) (1+ i))))))
  (iterate-lst dscs lst () 0))
