
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : tmdoc.scm
;; DESCRIPTION : generation of larger pieces of documentation
;; COPYRIGHT   : (C) 2002  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (tmdoc-down level)
  (cond ((equal? level 'title) 'chapter)
	((equal? level 'tmdoc-title) 'section)
	((equal? level 'chapter) 'section)
	((equal? level 'appendix) 'section)
	((equal? level 'section) 'subsection)
	((equal? level 'subsection) 'subsubsection)
	((equal? level 'subsubsection) 'paragraph)
	(else 'subparagraph)))

(define (tmdoc-branch x base-name level)
  (let* ((name (cadddr x))
	 (rel-name (relative-file-name base-name name)))
    (tmdoc-expand rel-name level)))

(define (tmdoc-substitute-sub l base-name)
  (if (null? l) l
      (cons (tmdoc-substitute (car l) base-name)
	    (tmdoc-substitute-sub (cdr l) base-name))))

(define (tmdoc-substitute x base-name)
  (cond ((match? x '(apply "hyper-link" 2))
	 (list 'apply "hyper-link" (caddr x)
	       (relative-file-name base-name (cadddr x))))
	((list? x) (cons (car x) (tmdoc-substitute-sub (cdr x) base-name)))
	(else x)))

(define (tmdoc-rewrite-one x base-name level)
  (cond ((func? x 'tmdoc-title) (cons level (cdr x)))
	((func? x 'tmdoc-license) '(document))
	((func? x 'traverse)
	 (cons 'document (tmdoc-rewrite (cdadr x) base-name level)))
	((match? x '(apply "branch" 2))
	 (tmdoc-branch x base-name (tmdoc-down level)))
	((match? x '(apply "extra-branch" 2))
	 (tmdoc-branch x base-name 'appendix))
	((match? x '(apply "tmdoc-copyright" *)) '(document))
	(else (tmdoc-substitute x base-name))))

(define (tmdoc-rewrite l base-name level)
  (if (null? l) l
      (let ((d1 (tmdoc-rewrite-one (car l) base-name level))
	    (d2 (tmdoc-rewrite (cdr l) base-name level)))
	(if (func? d1 'document) (append (cdr d1) d2) (cons d1 d2)))))

(define (tmdoc-expand file-name level)
  (let* ((t (load-file-as-tree "." file-name "TeXmacs"))
	 (u (cadr (assoc 'body (cdr (tree->object t))))))
    (cons 'document (tmdoc-rewrite (cdr u) file-name level))))

(define (tmdoc-expand-help file-name level)
  (let* ((body (tmdoc-expand file-name level))
	 (style (if (equal? level 'title) "tmmanual" "tmdoc"))
	 (doc `(document (style ,style) (body ,body))))
    (set-help-buffer file-name (object->tree doc))))

(define (tmdoc-expand-this level)
  (tmdoc-expand-help (get-name-buffer) level))
