
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : text-menu.scm
;; DESCRIPTION : menus for inserting structure in text mode
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Document headers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define letter-menu
  '(("Header" (make-big-expand "letter-header"))
    ---
    ("Opening" (make-big-expand "opening"))
    ("Closing" (make-big-expand "closing"))
    ("Signature" (make-big-expand "signature"))
    ---
    ("C.C." (make-big-expand "cc"))
    ("Encl." (make-big-expand "encl"))))

(define letter-header-menu
  '(("Address" (make-header-expand "address"))
    ("Date" (make-header-expand "letter-date"))
    ("Today" (begin (make-header-expand "letter-date") (make-date)))
    ("Destination" (make-header-expand "destination"))
    ("C.C." (make-header-expand "cc"))
    ("Encl." (make-header-expand "encl"))))

(define exam-header-menu
  '(("Class" (make-header-expand "class"))
    ("Date" (begin (go-end-of-header-element)
		   (make-expand-arg "title-date")))
    ("Title" (make-header-expand "title"))))

(define title-menu
  '((if (inside? "make-title")
	("Title" (make-header-expand "title"))
	("Author" (make-header-expand "author"))
	("Address" (make-header-expand "address"))
	("Email" (make-header-expand "title-email"))
	("Date" (make-header-expand "title-date"))
	---
	("TeXmacs notice" (make-expand "made-by-TeXmacs"))
	("Running title" (make-header-apply "header-title"))
	("Running author" (make-header-apply "header-author"))
	("Address block" (make-expand-arg "address-block"))
	("Today" (begin (make-header-expand "title-date")
			(make-date))))
    (if (and (not (inside? "make-title")) (inside? "abstract"))
	("Keywords" (make-section-arg "keywords"))
	("A.M.S. subject classification" (make-section-arg "AMS-class")))
    (if (and (not (inside? "make-title")) (not (inside? "abstract")))
	("Make title" (begin (make-big-expand "make-title")
			     (make-expand-arg "title")))
	("Abstract" (make-big-expand "abstract")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sections
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define chapter-menu
  '(("Chapter" (make-section-arg "chapter"))
    ("Appendix" (make-section-arg "appendix"))
    ("Prologue" (make-section "prologue"))
    ("Epilogue" (make-section "epilogue"))))

(define section-menu
  '(("Section" (make-section-arg "section"))
    ("Subsection" (make-section-arg "subsection"))
    ("Subsubsection" (make-section-arg "subsubsection"))
    ---
    ("Paragraph" (make-section-arg "paragraph"))
    ("Subparagraph" (make-section-arg "subparagraph"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Theorem like environments
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define environment-menu
  '((if (style-has? "header-exam-dtd")
	("Exercise" (make-big-expand "exercise"))
	("Problem" (make-big-expand "problem")))
    (if (not (style-has? "header-exam-dtd"))
	(if (style-has? "env-default-dtd")
	    ("Theorem" (make-big-expand "theorem"))
	    ("Proposition" (make-big-expand "proposition"))
	    ("Lemma" (make-big-expand "lemma"))
	    ("Corollary" (make-big-expand "corollary"))
	    ("Proof" (make-big-expand "proof"))
	    ("Axiom" (make-big-expand "axiom"))
	    ("Definition" (make-big-expand "definition"))
	    ---
	    ("Remark" (make-big-expand "remark"))
	    ("Note" (make-big-expand "note"))
	    ("Example" (make-big-expand "example"))
	    ("Warning" (make-big-expand "warning"))
	    ("Exercise" (make-big-expand "exercise"))
	    ("Problem" (make-big-expand "problem"))
	    ---)
	("Verbatim" (make-big-expand "verbatim"))
	("Code" (make-big-expand "code"))
	("Quote" (make-big-expand "quote"))
	("Quotation" (make-big-expand "quotation"))
	("Verse" (make-big-expand "verse")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Tags
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define content-tag-menu
  '(("Strong" (make-expand-arg "strong"))
    ("Emphasize" (make-expand-arg "em"))
    ("Definition" (make-expand-arg "dfn"))
    ("Sample" (make-expand-arg "samp"))
    ---
    ("Name" (make-expand-arg "name"))
    ("Person" (make-expand-arg "person"))
    ("Cite" (make-expand-arg "cite*"))
    ("Abbreviation" (make-expand-arg "abbr"))
    ("Acronym" (make-expand-arg "acronym"))
    ---
    ("Verbatim" (make-expand-arg "verbatim"))
    ("Keyboard" (make-expand-arg "kbd"))
    ("Code" (make-expand-arg "code*"))
    ("Variable" (make-expand-arg "var"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Enumerations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define itemize-menu
  '(("Default" (make-list "itemize"))
    ---
    ("Bullets" (make-list "itemize-dot"))
    ("Dashes" (make-list "itemize-minus"))
    ("Arrows" (make-list "itemize-arrow"))))

(define enumerate-menu
  '(("Default" (make-list "enumerate"))
    ---
    ("1, 2, 3, ..." (make-list "enumerate-numeric"))
    ("i, ii, iii, ..." (make-list "enumerate-roman"))
    ("I, II, III, ..." (make-list "enumerate-Roman"))
    ("a, b, c, ..." (make-list "enumerate-alpha"))
    ("A, B, C, ..." (make-list "enumerate-Alpha"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Automatically generated content
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define automatic-menu
  '(("Table of contents" (make-aux "table-of-contents" "toc"))
    ("Bibliography" ... (interactive
      '("Bibliography style:" "Bibliography file:") 'make-bib))
    ("Index" (make-aux "the-index" "idx"))
    ("Glossary" (make-aux "the-glossary" "gly"))
    ;;	    ("List of figures" (make-aux "list-of-figures" "lof"))
    ;;	    ("List of tables" (make-aux "list-of-tables" "lot"))
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inserting mathematical markup into the text
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define insert-mathematics-menu
  '(("Formula" "$" (make-with "mode" "math"))
    (if (style-has? "env-math-dtd")
	("Equation" (begin (make-big-expand "equation*") (temp-proof-fix)))
	("Equations" (begin (make-expand-arg "eqnarray*") (temp-proof-fix)))
	("Numbered equation"
	 (begin (make-big-expand "equation") (temp-proof-fix))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inserting extern sessions into the text
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define session-menu '()) ;; each computer algebra system appends its own entry

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Style dependent menus
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define text-menu
  '((if (style-has? "header-title-dtd")
	(-> "Title" (link title-menu)))
    (if (style-has? "header-letter-dtd")
	(if (not (inside? "letter-header"))
	    (-> "Header" (link letter-menu)))
	(if (inside? "letter-header")
	    (-> "Letter" (link letter-header-menu))))
    (if (style-has? "header-exam-dtd")
	(-> "Header" (link exam-header-menu)))
    (if (style-has? "book-style")
	(-> "Chapter" (link chapter-menu)))
    (if (and (style-has? "section-latex-dtd")
	     (not (style-has? "header-exam-dtd")))
	(-> "Section" (link section-menu)))
    (if (style-has? "std-markup-dtd")
	(-> "Environment" (link environment-menu))
	(-> "Content tag" (link content-tag-menu)))
    (if (style-has? "common-base-dtd")
	---)
    (if (style-has? "std-list-dtd")
	(if (or (inside-list?) (inside-description?))
	    ("New item" (make-item)))
	(-> "Itemize" (link itemize-menu))
        (-> "Enumerate" (link enumerate-menu))
	(-> "Description"
	    ("Default"  (make-list "description")))
	---)
    (-> "Mathematics" (link insert-mathematics-menu))
    (if (style-has? "program-dtd")
	(-> "Session"
	    (link session-menu)
	    ---
	    ("Other" ... (interactive
              '("Session type:" "Session name:") 'make-session))))
    (if (style-has? "section-automatic-dtd")
	(-> "Automatic" (link automatic-menu)))))
