
/******************************************************************************
* MODULE     : tm_menus.gen.cc
* DESCRIPTION: Dynamical menus
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "tm_layout.hh"
#include "Widget/make_widget.hh"
#include "tm_buffer.hh"

widget make_menu_widget (server sv, scheme_tree p, bool active= true);
scheme_tree menu_merge (scheme_tree p, scheme_tree q);

/******************************************************************************
* Constructor and destructor
******************************************************************************/

tm_layout_rep::tm_layout_rep () {}
tm_layout_rep::~tm_layout_rep () {}

/******************************************************************************
* Compilation routines
******************************************************************************/

class server_command_rep: public command_rep {
  server  sv;
  scheme_tree p;
public:
  server_command_rep (server sv2, scheme_tree p2): sv (sv2), p(p2) {}
  void apply () {
    sv->get_editor()->buf->mark_undo_block ();
    sv->get_editor()->set_input_normal ();
    sv->exec_delayed (p);
    sv->get_editor()->notify_change (THE_DECORATIONS);
  }
  ostream& print (ostream& out) { return out << p; }
};

command tm_layout_rep::make_command (scheme_tree p) {
  // cout << "tm_layout_rep::make_command : " << p << "\n";
  return new server_command_rep (this, p);
}

/******************************************************************************
* Abstract function for creating menu widgets
******************************************************************************/

class make_menu_widget_rep: public make_widget_rep {
  server  sv;
  scheme_tree p;
  bool active;
public:
  make_menu_widget_rep (server sv2, scheme_tree p2, bool active2):
    sv(sv2), p(p2), active (active2) {}
  ostream& print (ostream& out) { return out << p; }
  widget get_widget (display dis);
};

widget
make_menu_widget_rep::get_widget (display dis) {
  return make_menu_widget (sv, p, active);
}

make_widget
make_menu_make_widget (server sv, scheme_tree p, bool active) {
  return new make_menu_widget_rep (sv, p, active);
}

/******************************************************************************
* Subroutines
******************************************************************************/

static inline bool
is_quoted (tree t) {
  return is_atomic (t) && is_quoted (as_string (t));
}

static inline string
unquote (tree t) {
  return unquote (as_string (t));
}

widget
make_menu_widget (server sv, scheme_tree p, bool active) {
  scheme_tree t= tuple ("make-menu-widget",
			tuple ("quote", p),
			active ? string ("#t") : string ("#f"));
  widget w;
  if (sv->eval_widget (t, w)) {
    cerr << "TeXmacs] error in make-menu-widget " << p << "\n";
    array<widget> a (0);
    return horizontal_array (a); }
  return w;
}

/******************************************************************************
* Operations on menus
******************************************************************************/

static void
menu_append (scheme_tree& a, scheme_tree p) {
  if (is_tuple (a) && is_tuple (p)) {
    int i, k= N(a), n= N(p);

    if (((n == 2) || (n == 3)) && is_quoted (p[0]))
      for (i=0; i<k; i++)
	if (is_tuple (a[i]) && ((N(a[i]) == 2) || (N(a[i]) == 3)) &&
	    (a[i][0]==p[0]))
	  {
	    a[i]= p;
	    return;
	  }

    if ((is_tuple (p, "=>") || is_tuple (p, "->")) &&
	(n >= 2) && is_atomic (p[1]))
      for (i=0; i<k; i++)
	if (is_tuple (a[i]) && (N(a[i]) >= 2) &&
	    (a[i][0]==p[0]) && (a[i][1]==p[1]))
	  {
	    int j;
	    scheme_tree b (TUPLE, n-2);
	    for (j=0; j<n-2; j++) b[j]= p[j+2];
	    a[i]= menu_merge (a[i], scheme_tree (b));
	    return;
	  }
  }

  a << p;
}

scheme_tree
menu_merge (scheme_tree p, scheme_tree q) {
  if (is_atomic (p)) return scheme_tree (TUPLE);
  else {
    int i, k= N(p);
    scheme_tree a (TUPLE, k);
    for (i=0; i<k; i++) a[i]= p[i];

    if (is_atomic (q)) menu_append (a, q);
    else {
      int n= N(q);
      if ((n > 0) && ((q[0] == "---") || (q[0] == "|") || is_tuple (q[0])))
	for (i=0; i<n; i++) menu_append (a, q[i]);
      else menu_append (a, q);
    }

    return a;
  }
}

/******************************************************************************
* Properties of the current window
******************************************************************************/

void
tm_layout_rep::set_window_property (scheme_tree what, scheme_tree val) {
  tm_widget meta= get_meta ();
  meta->props (what)= val;
}

void
tm_layout_rep::set_bool_window_property (string what, bool val) {
  tm_widget meta= get_meta ();
  meta->props (what)= (val? string ("true"): string ("false"));
}

void
tm_layout_rep::set_int_window_property (string what, int val) {
  tm_widget meta= get_meta ();
  meta->props (what)= as_tree (val);
}

void
tm_layout_rep::set_string_window_property (string what, string val) {
  tm_widget meta= get_meta ();
  meta->props (what)= val;
}

scheme_tree
tm_layout_rep::get_window_property (scheme_tree what) {
  tm_widget meta= get_meta ();
  return meta->props [what];
}

bool
tm_layout_rep::get_bool_window_property (string what) {
  tm_widget meta= get_meta ();
  return as_bool (meta->props [what]);
}

int
tm_layout_rep::get_int_window_property (string what) {
  tm_widget meta= get_meta ();
  return as_int (meta->props [what]);
}

string
tm_layout_rep::get_string_window_property (string what) {
  tm_widget meta= get_meta ();
  return as_string (meta->props [what]);
}

/******************************************************************************
* Interface
******************************************************************************/

static string
icon_bar_name (int which) {
  if (which == 0) return "main";
  else if (which == 1) return "context";
  else return "user";
}

void
tm_layout_rep::menu_widget (scheme_tree p, widget& w) {
  tm_widget meta= get_meta ();
  w= make_menu_widget (server (this), p);
}

void
tm_layout_rep::menu_main (scheme_tree p) {
  tm_widget meta= get_meta ();
  widget w= make_menu_widget (server (this), p);
  meta->set_subwidget (((widget) meta) ["header"] ["menu"] ["bar"], "menu", w);
}

void
tm_layout_rep::menu_icons (int which, scheme_tree p) {
  if ((which<0) || (which>2)) return;
  string name= icon_bar_name (which);
  tm_widget meta= get_meta ();
  widget w= make_menu_widget (server (this), p);
  meta->set_subwidget (((widget) meta) ["header"] [name] ["bar"], "icons", w);
}

void
tm_layout_rep::show_header (bool flag) {
  if (!has_view ()) return;
  tm_widget meta= get_meta ();
  meta->set_subwidget_flag (((widget) meta) ["header"], flag);
}

void
tm_layout_rep::show_icon_bar (int which, bool flag) {
  if ((which<0) || (which>2) || (!has_view())) return;
  string name= icon_bar_name (which);
  tm_widget meta= get_meta ();
  meta->set_subwidget_flag (((widget) meta) ["header"] [name], flag);
}

void
tm_layout_rep::show_footer (bool flag) {
  if (!has_view ()) return;
  tm_widget meta= get_meta ();
  meta->set_footer_flag (flag);
}

bool
tm_layout_rep::visible_header () {
  tm_widget meta= get_meta ();
  return meta->get_subwidget_flag (((widget) meta) ["header"]);
}

bool
tm_layout_rep::visible_icon_bar (int which) {
  if ((which<0) || (which>2)) return false;
  string name= icon_bar_name (which);
  tm_widget meta= get_meta ();
  return meta->get_subwidget_flag (((widget) meta) ["header"] [name]);
}

bool
tm_layout_rep::visible_footer () {
  tm_widget meta= get_meta ();
  return meta->get_footer_flag ();
}

void
tm_layout_rep::set_shrinking_factor (int sf) {
  if (!has_view ()) return;
  tm_widget meta= get_meta ();
  meta->set_shrinking_factor (sf);
}

int
tm_layout_rep::get_shrinking_factor () {
  tm_widget meta= get_meta ();
  return meta->get_shrinking_factor ();
}
